@@ -22,16 +22,18 @@
 
 G_BEGIN_DECLS;
 
-void goto_tab_1 (tilda_window *tw);
-void goto_tab_2 (tilda_window *tw);
-void goto_tab_3 (tilda_window *tw);
-void goto_tab_4 (tilda_window *tw);
-void goto_tab_5 (tilda_window *tw);
-void goto_tab_6 (tilda_window *tw);
-void goto_tab_7 (tilda_window *tw);
-void goto_tab_8 (tilda_window *tw);
-void goto_tab_9 (tilda_window *tw);
-void goto_tab_10 (tilda_window *tw);
+gboolean goto_tab_1 (tilda_window *tw);
+gboolean goto_tab_2 (tilda_window *tw);
+gboolean goto_tab_3 (tilda_window *tw);
+gboolean goto_tab_4 (tilda_window *tw);
+gboolean goto_tab_5 (tilda_window *tw);
+gboolean goto_tab_6 (tilda_window *tw);
+gboolean goto_tab_7 (tilda_window *tw);
+gboolean goto_tab_8 (tilda_window *tw);
+gboolean goto_tab_9 (tilda_window *tw);
+gboolean goto_tab_10 (tilda_window *tw);
+void ccopy (tilda_window *tw);
+void cpaste (tilda_window *tw);
 void prev_tab (tilda_window *tw);
 void next_tab (tilda_window *tw);
 char* get_window_title (GtkWidget *widget, tilda_window *tw);
@@ -40,7 +42,6 @@ void window_title_changed (GtkWidget *widget, gpointer data);
 void clean_up_no_args ();
 void free_and_remove (tilda_window *tw);
 void clean_up (tilda_window *tw);
-void clean_up_no_main (tilda_window *tw);
 void close_tab_on_exit (GtkWidget *widget, gpointer data);
 void icon_title_changed (GtkWidget *widget, gpointer win);
 void commit (VteTerminal *vteterminal, gpointer user_data);
