@@ -33,13 +33,15 @@
 #include "key_grabber.h"
 #include "../tilda-config.h"
 
-Display *dpy;
-Window root;
-Window win;
-Window termwin;
-Window last_focused;
-int screen;
-KeySym key;
+static Display *dpy;
+static Window root;
+static Window win;
+static Window termwin;
+static Window last_focused;
+static int screen;
+static KeySym key;
+
+static unsigned int display_width, display_height;
 
 /*
  * The slide positions are derived from FVMW sources, file fvmm/move_resize.c,
@@ -99,7 +101,7 @@ void generate_animation_positions(struct tilda_window_ *tw)
     }
 }
 
-void pull (struct tilda_window_ *tw)
+static void pull_state (struct tilda_window_ *tw, int state)
 {
 #ifdef DEBUG
     puts("pull");
@@ -109,7 +111,7 @@ void pull (struct tilda_window_ *tw)
     gint w, h;
     static gint pos=0;
     
-    if (pos == 0)
+    if (pos == 0 && state != PULL_UP)
     {
         gdk_threads_enter();
 
@@ -156,7 +158,7 @@ void pull (struct tilda_window_ *tw)
         gdk_flush ();
         gdk_threads_leave();
     }
-    else
+    else if (state != PULL_DOWN)
     {
         gdk_threads_enter();
 
@@ -192,7 +194,13 @@ void pull (struct tilda_window_ *tw)
     }
 }
 
-void key_grab (tilda_window *tw)
+//  Wrapper function so pull() calls without a state toggle like they used to
+void pull (struct tilda_window_ *tw)
+{
+    pull_state (tw, PULL_TOGGLE);
+}
+
+static void key_grab (tilda_window *tw)
 {
 #ifdef DEBUG
     puts("key_grab");
@@ -265,17 +273,17 @@ void *wait_for_signal (tilda_window *tw)
     
     key_grab (tw);
 
-    if (cfg_getbool (tw->tc, "down"))
-        pull (tw);
-    else
+    if (cfg_getbool (tw->tc, "hidden"))
     {
         /*
          * Fixes bug that causes Tilda to eat up 100% of CPU 
          * if set to stay hidden when started
          */        
-        pull (tw);
-        pull (tw);
+        pull_state(tw, PULL_DOWN);
+        pull_state(tw, PULL_UP);
     }
+    else
+        pull_state(tw, PULL_DOWN);
 
     for (;;)
     {
