@@ -23,6 +23,10 @@
 
 G_BEGIN_DECLS;
 
+#define PULL_TOGGLE 0
+#define PULL_DOWN 1
+#define PULL_UP 2
+
 void pull (tilda_window *tw);
 void *wait_for_signal (tilda_window *tw);
 void generate_animation_positions (tilda_window *tw);
