@@ -32,6 +32,11 @@
 #include <fontconfig/fontconfig.h>
 #endif
 
+#include <X11/Xlib.h>
+#include <X11/Xlibint.h>
+#include <X11/Xproto.h>
+#include <X11/extensions/Xrandr.h>
+
 #include <vte/vte.h>
 #include "config.h"
 #include "tilda.h"
@@ -41,12 +46,9 @@
 #include "key_grabber.h"
 #include "wizard.h"
 
-/* unneeded right now (not used) */
-/* gchar *user, *display; */
-
 static tilda_window *tw;
 
-void pull_no_args()
+static void pull_no_args()
 {
     pull(tw);
 }
@@ -68,16 +70,20 @@ void print_and_exit (gchar *message, gint exitval)
  *
  * @param tw the tilda_window structure for this instance
  */
-gint getnextinstance (tilda_window *tw)
+static gint getnextinstance (tilda_window *tw)
 {
 #ifdef DEBUG
     puts("getnextinstance");
 #endif
 
     gchar *lock_dir;
+    gchar *name;
+    gchar **tokens;
     gchar *lock_subdir = "/.tilda/locks";
     gint lock_dir_size = 0;
     gint count = 0;
+    gint current = 0;
+    gint tmp = 0;
     GDir *dir;
 
     /* Figure out the size of the lock_dir variable. Make sure to
@@ -92,10 +98,25 @@ gint getnextinstance (tilda_window *tw)
     g_snprintf (lock_dir, lock_dir_size, "%s%s", tw->home_dir, lock_subdir);
     dir = g_dir_open (lock_dir, 0, NULL);
 
-    /* FIXME: check that this name corresponds to a valid lock */
-    /* FIXME: check the last char(s) to find out the correct next instance */
-    while (dir != NULL && g_dir_read_name (dir))
-        count++;
+    while (dir != NULL && (name = (gchar*)g_dir_read_name (dir)))
+    {
+        tokens = g_strsplit (name, "_", 3);
+
+        if (tokens != NULL)
+        {
+            current = atoi (tokens[2]);
+            g_strfreev (tokens);
+
+            if (current - tmp > 1) {
+                count = tmp + 1;
+                break;
+            }
+
+            count++;
+
+            tmp = current;
+        }
+    }
 
     /* Free memory that we allocated */
     if (dir != NULL)
@@ -159,7 +180,7 @@ void getinstance (tilda_window *tw)
  * @param lock_pid the pid of the lock file (if it was valid)
  * @return TRUE if this is a lockfile, FALSE otherwise
  */
-gboolean islockfile (gchar *filename, gint *lock_pid)
+static gboolean islockfile (gchar *filename, gint *lock_pid)
 {
 #ifdef DEBUG
     puts("islockfile");
@@ -196,7 +217,7 @@ gboolean islockfile (gchar *filename, gint *lock_pid)
  *
  * @param tw the tilda_window structure for this instance
  */
-void clean_tmp (tilda_window *tw)
+static void clean_tmp (tilda_window *tw)
 {
 #ifdef DEBUG
     puts("clean_tmp");
@@ -285,7 +306,7 @@ void clean_tmp (tilda_window *tw)
 
     if (dir != NULL)
         g_dir_close (dir);
-    
+
     free (running_pids);
     free (lock_dir);
 }
@@ -299,7 +320,7 @@ void clean_tmp (tilda_window *tw)
  * @param tw the tilda_window from main. Used for setting values in tw->tc.
  * @param tt the tilda_term, from main. Used for calling the wizard.
  */
-void parse_cli (int *argc, char ***argv, tilda_window *tw, tilda_term *tt)
+static void parse_cli (int *argc, char ***argv, tilda_window *tw, tilda_term *tt)
 {
 #ifdef DEBUG
     puts("parse_cli");
@@ -321,12 +342,14 @@ void parse_cli (int *argc, char ***argv, tilda_window *tw, tilda_term *tt)
     gboolean scrollbar = cfg_getbool (tw->tc, "scrollbar");
     gboolean show_config = FALSE;
     gboolean version = FALSE;
+    gboolean hidden = cfg_getbool (tw->tc, "hidden");
 
     /* All of the various command-line options */
     GOptionEntry cl_opts[] = {
         { "antialias",          'a', 0, G_OPTION_ARG_NONE,      &antialias,         "Use Antialiased Fonts", NULL },
         { "background-color",   'b', 0, G_OPTION_ARG_STRING,    &background_color,  "Set the background color", NULL },
         { "command",            'c', 0, G_OPTION_ARG_STRING,    &command,           "Run a command at startup", NULL },
+        { "hidden",             'h', 0, G_OPTION_ARG_NONE,      &hidden,            "Start Tilda hidden", NULL },
         { "font",               'f', 0, G_OPTION_ARG_STRING,    &font,              "Set the font to the following string", NULL },
         { "lines",              'l', 0, G_OPTION_ARG_INT,       &lines,             "Scrollback Lines", NULL },
         { "scrollbar",          's', 0, G_OPTION_ARG_NONE,      &scrollbar,         "Use Scrollbar", NULL },
@@ -360,25 +383,25 @@ void parse_cli (int *argc, char ***argv, tilda_window *tw, tilda_term *tt)
         printf ("to see all possible options.\n\n");
 
         printf ("Error message: %s\n", error->message);
-        
+
         exit (1);
     }
-    
+
     /* If we need to show the version, show it then exit normally */
     if (version)
     {
         printf ("%s\n\n", TILDA_VERSION);
-        
+
         printf ("Copyright (c) 2005,2006 Tristan Sloughter (sloutri@iit.edu)\n");
         printf ("Copyright (c) 2005,2006 Ira W. Snyder (tilda@irasnyder.com)\n\n");
-        
+
         printf ("This program comes with ABSOLUTELY NO WARRANTY.\n");
         printf ("This is free software, and you are welcome to redistribute it\n");
         printf ("under certain conditions. See the file COPYING for details.\n");
 
         exit (0);
     }
-    
+
     /* Now set the options in the config, if they changed */
     if (background_color != cfg_getstr (tw->tc, "background_color"))
         cfg_setstr (tw->tc, "background_color", background_color);
@@ -394,7 +417,10 @@ void parse_cli (int *argc, char ***argv, tilda_window *tw, tilda_term *tt)
     if (lines != cfg_getint (tw->tc, "lines"))
         cfg_setint (tw->tc, "lines", lines);
     if (transparency != cfg_getint (tw->tc, "transparency"))
+    {
+        cfg_setbool (tw->tc, "enable_transparency", transparency);
         cfg_setint (tw->tc, "transparency", transparency);
+    }
     if (x_pos != cfg_getint (tw->tc, "x_pos"))
         cfg_setint (tw->tc, "x_pos", x_pos);
     if (y_pos != cfg_getint (tw->tc, "y_pos"))
@@ -402,16 +428,245 @@ void parse_cli (int *argc, char ***argv, tilda_window *tw, tilda_term *tt)
 
     if (antialias != cfg_getbool (tw->tc, "antialias"))
         cfg_setbool (tw->tc, "antialias", antialias);
+    if (hidden != cfg_getbool (tw->tc, "hidden"))
+        cfg_setbool (tw->tc, "hidden", hidden);
     if (scrollbar != cfg_getbool (tw->tc, "scrollbar"))
         cfg_setbool (tw->tc, "scrollbar", scrollbar);
 
     /* Show the config wizard, if it was requested */
     if (show_config)
-        if ((wizard (*argc, *argv, tw, tt)) == 1) { 
-          clean_up(tw); 
+        if ((wizard (*argc, *argv, tw, tt)) == 1) {
+          clean_up(tw);
         }
 }
 
+int get_display_dimension (int dimension)
+{
+    Display                 *dpy;
+    XRRScreenSize           *sizes;
+    XRRScreenConfiguration  *sc;
+    Window                  root;
+    int                     nsize;
+    int                     screen = -1;
+    char                    *display_name = NULL;
+
+    static int height = -1;
+    static int width  = -1;
+
+/* Turn this on by default, if we need to turn if off for some people, we
+ * should probably add a configure option, such as
+ * --enable-cache-display-dimensions or something similar. */
+#define CACHE_DISPLAY_DIMENSIONS
+#ifdef CACHE_DISPLAY_DIMENSIONS
+    if (height != -1 && dimension == HEIGHT)
+        return height;
+
+    if (width != -1 && dimension == WIDTH)
+        return width;
+#endif
+
+    dpy = XOpenDisplay (display_name);
+
+    if (dpy == NULL)
+    {
+        /* We can't just exit, so print a message and don't attempt to
+         * do anything since we won't know what to do! */
+        fprintf (stderr, "Can't open display %s\n", XDisplayName(display_name));
+        return;
+    }
+
+    screen = DefaultScreen (dpy);
+    root = RootWindow (dpy, screen);
+    sc = XRRGetScreenInfo (dpy, root);
+
+    if (sc == NULL)
+    {
+        /* We can't just exit, so print a message and don't attempt to
+         * do anything since we don't know what to do. */
+        fprintf (stderr, "Can't get screen info\n");
+    }
+
+    sizes = XRRConfigSizes(sc, &nsize);
+
+    XRRFreeScreenConfigInfo(sc);
+    XCloseDisplay (dpy);
+
+    height = sizes->height;
+    width  = sizes->width;
+
+    if (dimension == HEIGHT)
+        return height;
+
+    if (dimension == WIDTH)
+        return width;
+
+    return -1; // bad choice
+}
+
+/*
+ * Finds the coordinate that will center the tilda window in the screen.
+ *
+ * If you want to center the tilda window on the top or bottom of the screen,
+ * pass the screen width into screen_dimension and the tilda window's width
+ * into the tilda_dimension variable. The result will be the x coordinate that
+ * should be used in order to have the tilda window centered on the screen.
+ *
+ * Centering based on y coordinate is similar, just use the screen height and
+ * tilda window height.
+ */
+int find_centering_coordinate (const int screen_dimension, const int tilda_dimension)
+{
+#ifdef DEBUG
+    puts ("find_centering_coordinate");
+#endif
+
+    const float screen_center = screen_dimension / 2.0;
+    const float tilda_center  = tilda_dimension  / 2.0;
+
+    return screen_center - tilda_center;
+}
+
+int write_config_file (tilda_window *tw)
+{
+    FILE *fp;
+
+    /* Check to see if writing is disabled. Leave early if it is. */
+    if (tw->config_writing_disabled)
+        return 1;
+
+    fp = fopen(tw->config_file, "w");
+
+    if (fp != NULL)
+    {
+        cfg_print (tw->tc, fp);
+
+        if (fclose (fp) != 0)
+        {
+            // An error occurred
+            perror ("tilda error");
+            fprintf (stderr, "Unable to close the config file\n");
+        }
+
+    }
+    else
+    {
+        perror ("tilda error");
+        fprintf (stderr, "Unable to write the config file to %s\n", tw->config_file);
+    }
+
+    return 0;
+}
+
+/*
+ * Compares two config versions together.
+ *
+ * Returns -1 if config1 is older than config2 (UPDATE REQUIRED)
+ * Returns  0 if config1 is equal to   config2 (NORMAL USAGE)
+ * Returns  1 if config1 is newer than config2 (DISABLE WRITE)
+ */
+static gboolean compare_config_versions (const gchar *config1, const gchar *config2)
+{
+    /*
+     * 1) Split apart both strings using the .'s
+     * 2) Compare the major-major version
+     * 3) Compare the major version
+     * 4) Compare the minor version
+     */
+
+    gchar **config1_tokens;
+    gchar **config2_tokens;
+    gint  config1_version[3];
+    gint  config2_version[3];
+    gint  i;
+
+    config1_tokens = g_strsplit (config1, ".", 3);
+    config2_tokens = g_strsplit (config2, ".", 3);
+
+    for (i=0; i<3; i++)
+    {
+        config1_version[i] = atoi (config1_tokens[i]);
+        config2_version[i] = atoi (config2_tokens[i]);
+    }
+
+    g_strfreev (config1_tokens);
+    g_strfreev (config2_tokens);
+
+    /* We're done splitting things, so compare now */
+    for (i=0; i<3; i++)
+    {
+        if (config1_version[i] > config2_version[i])
+            return CONFIG1_NEWER;
+
+        if (config1_version[i] < config2_version[i])
+            return CONFIG1_OLDER;
+    }
+
+    return CONFIGS_SAME;
+}
+
+/*
+ * Compare config file versions and see if we know about a new version.
+ *
+ * If we have a version newer than what we know about, we need to stop
+ * anything from writing to the config file.
+ *
+ * If we are at the same config version, we're done, so exit early.
+ *
+ * If the config is older than we are now, update it and then write
+ * it to disk.
+ */
+static void try_to_update_config_file (tilda_window *tw)
+{
+    gchar *current_config = cfg_getstr (tw->tc, "tilda_config_version");
+
+    if (compare_config_versions (current_config, PACKAGE_VERSION) == CONFIGS_SAME)
+        return; // Same version as ourselves, we're done!
+
+    if (compare_config_versions (current_config, PACKAGE_VERSION) == CONFIG1_NEWER)
+    {
+        // We have a version newer than ourselves!
+        // Disable writing to the config for safety.
+        tw->config_writing_disabled = TRUE;
+        return; // Out early, since we won't be able to update anyway!
+    }
+
+    /* NOTE: Start with the oldest config version first, and work our way up to the
+     * NOTE: newest that we support, updating our current version each time.
+     *
+     * NOTE: You may need to re-read the config each time! Probably not though,
+     * NOTE: since you should be updating VALUES not names directly in the config.
+     * NOTE: Try to rely on libconfuse to generate the configs :) */
+
+    /* Below is a template for creating new entries in the updater. If you ever
+     * change anything between versions, copy this, replacing YOUR_VERSION
+     * with the new version that you are making. */
+#if 0
+    if (compare_config_versions (current_config, YOUR_VERSION) == CONFIG1_OLDER)
+    {
+        // TODO: Add things here to migrate from whatever we are to YOUR_VERSION
+        current_config = YOUR_VERSION;
+    }
+#endif
+
+    if (compare_config_versions (current_config, "0.09.4") == CONFIG1_OLDER)
+    {
+        /* Nothing to update here. All we did was add an option, there is no
+         * need to rewrite the config file here, since the writer at the end
+         * will automatically add the default value of the new option. */
+        current_config = "0.09.4";
+    }
+
+    /* We've run through all the updates, so set our config file version to the
+     * version we're at now, then write out the config file.
+     *
+     * NOTE: this only happens if we upgraded the config, due to some early-exit
+     * logic above.
+     */
+
+    cfg_setstr (tw->tc, "tilda_config_version", current_config);
+    write_config_file (tw);
+}
+
 int main (int argc, char **argv)
 {
 #ifdef DEBUG
@@ -439,6 +694,9 @@ int main (int argc, char **argv)
     /* Set: tw->instance, tw->config_file, and parse the config file */
     init_tilda_window_instance (tw);
 
+    /* Check if the config file is old. If it is, update it */
+    try_to_update_config_file (tw);
+
 #ifdef DEBUG
     /* Have to do this early. */
     if (getenv ("VTE_PROFILE_MEMORY"))
